<?php

namespace Adldap\Configuration;

use Adldap\AdldapException;

/**
 * Class ConfigurationException.
 *
 * Thrown when a configuration value does not exist, or a
 * configuration value being set is not valid.
 */
class ConfigurationException extends AdldapException
{
    //
}
